const { cloneDeep, get } = require('lodash')
const fixturesService = require('pubsweet-component-fixture-service')
const ah = require('../config/authsome-helpers')

describe('Authsome Helpers', () => {
  let testFixtures = {}
  beforeEach(() => {
    testFixtures = cloneDeep(fixturesService.fixtures)
  })
  it('stripeCollection - should return collection', () => {
    const { collection } = testFixtures.collections
    const result = ah.stripeCollectionByRole(collection)
    expect(result).toBeTruthy()
  })
  it('stripeFragment - should return fragment', () => {
    const { fragment } = testFixtures.fragments
    const result = ah.stripeFragmentByRole({ fragment })
    expect(result).toBeTruthy()
  })

  it('stripeCollection - author should not see accepted HE name before recommendation made', () => {
    const { collection } = testFixtures.collections
    collection.status = 'underReview'
    collection.handlingEditor = {
      ...collection.handlingEditor,
      isAccepted: true,
    }

    const result = ah.stripeCollectionByRole(collection, 'author')
    const { handlingEditor = {} } = result

    expect(handlingEditor.email).toBeFalsy()
    expect(handlingEditor.name).toEqual('Assigned')
  })

  it('stripeCollection - author should not see Assigned until HE accepted ', () => {
    const { collection } = testFixtures.collections
    collection.status = 'underReview'
    collection.handlingEditor = {
      ...collection.handlingEditor,
      isAccepted: false,
    }

    const result = ah.stripeCollectionByRole(collection, 'author')
    const { handlingEditor = {} } = result

    expect(handlingEditor).toBeFalsy()
    expect(handlingEditor.name).not.toEqual('Assigned')
  })

  it('stripeCollection - author should see HE name after recommendation made', () => {
    const { collection } = testFixtures.collections
    collection.status = 'revisionRequested'

    const result = ah.stripeCollectionByRole(collection, 'author')
    const { handlingEditor = {} } = result

    expect(handlingEditor.name).not.toEqual('Assigned')
  })

  it('stripeCollection - other user than author should see HE name before recommendation made', () => {
    const { collection } = testFixtures.collections
    collection.status = 'underReview'

    const result = ah.stripeCollectionByRole(collection, 'admin')
    const { handlingEditor = {} } = result

    expect(handlingEditor.name).not.toEqual('Assigned')
  })

  it('stripeCollection - other user than author should see HE name after recommendation made', () => {
    const { collection } = testFixtures.collections
    collection.status = 'revisionRequested'

    const result = ah.stripeCollectionByRole(collection, 'admin')
    const { handlingEditor = {} } = result

    expect(handlingEditor.name).not.toEqual('Assigned')
  })

  it('stripeCollection - returns if collection does not have HE', () => {
    const { collection } = testFixtures.collections
    delete collection.handlingEditor

    const result = ah.stripeCollectionByRole(collection, 'admin')
    expect(result.handlingEditor).toBeFalsy()
  })

  it('stripeFragment - reviewer should not see authors email', () => {
    const { fragment } = testFixtures.fragments
    const result = ah.stripeFragmentByRole({ fragment, role: 'reviewer' })
    const { authors = [] } = result
    expect(authors[0].email).toBeFalsy()
  })
  it('stripeFragment - other roles than reviewer should see authors emails', () => {
    const { fragment } = testFixtures.fragments
    const result = ah.stripeFragmentByRole({ fragment, role: 'author' })
    const { authors = [] } = result

    expect(authors[0].email).toBeTruthy()
  })

  it('stripeFragment - reviewer should not see cover letter', () => {
    const { fragment } = testFixtures.fragments
    const result = ah.stripeFragmentByRole({ fragment, role: 'reviewer' })
    const { files = {} } = result
    expect(files.coverLetter).toBeFalsy()
  })
  it('stripeFragment - reviewer should not see others reviews', () => {
    const { fragment } = testFixtures.fragments
    const result = ah.stripeFragmentByRole({ fragment, role: 'reviewer' })
    const { recommendations } = result
    expect(recommendations).toEqual([])
  })

  it('stripeFragment - author should not see recommendations if a decision has not been made', () => {
    const { fragment } = testFixtures.fragments
    fragment.recommendations = [
      {
        comments: [
          {
            content: 'private',
            public: false,
          },
          {
            content: 'public',
            public: true,
          },
        ],
      },
    ]
    const { recommendations } = ah.stripeFragmentByRole({
      fragment,
      role: 'author',
      status: 'underReview',
    })
    expect(recommendations).toHaveLength(0)
  })
  it('stripeFragment - author should see reviews only if recommendation has been made and only public ones', () => {
    const { fragment } = testFixtures.fragments
    fragment.recommendations = [
      {
        comments: [
          {
            content: 'private',
            public: false,
          },
          {
            content: 'public',
            public: true,
          },
        ],
      },
    ]
    const result = ah.stripeFragmentByRole({
      fragment,
      role: 'author',
      status: 'revisionRequested',
    })
    const publicComments = get(result, 'recommendations[0].comments')
    expect(publicComments).toHaveLength(1)
  })
})
